#include <Arduino.h>

#include "commands.h"
#include "utils.h"
#include "globals.h"
#include "constants.h"
#include "clock.h"


void updateDelay() {
  int flag;
  waitCharacter();
  flag = Serial.read();
  switch(flag) {
    case 'C':
      waitChoice = CALIBRATION_LOOP;
      break;
    case 'N':
      waitChoice = NORMAL_LOOP;
      break;
    default:
      break;
  }
}

void updateNumberOfLeds() {
  int n;
  n = Serial.read();
  switch(n) {
    case '1':
      numberOfLeds = 1;
      break;
    case '2':
      if (numberOfLeds != 2) {
        // reset part status
        secondPart = false;
      }
      numberOfLeds = 2;
      break;
    case 'T':
      secondPart = false;
      numberOfLeds = 0;  
    default:
      break;
      // do nothing
  }
  Serial.print("# Updated number of leds to ");
  Serial.println(numberOfLeds);
}


void parseDateTime() {
  int year = Serial.parseInt();
  Serial.read(); // skip -
  int month = Serial.parseInt();
  Serial.read(); // skip -
  int day = Serial.parseInt();
  Serial.read(); // skip T
  int hour = Serial.parseInt();
  Serial.read(); // skip :
  int minutes = Serial.parseInt();
  Serial.read(); // skip :
  int seconds = Serial.parseInt();

  Serial.print("# New date will be ");
  Serial.print(year);
  Serial.print("-");
  Serial.print(month);
  Serial.print("-");
  Serial.print(day);
  Serial.print("T");
  Serial.print(hour);
  Serial.print(":");
  Serial.print(minutes);
  Serial.print(":");
  Serial.println(seconds);

  setDateTime(year, (char)month, (char)day, (char)hour, (char)minutes, (char)seconds);
}

void handleCommands() {
  int command;
  if (!Serial.available()) {
    return;
  }
  command = Serial.read();

  switch( command ) {
    case 'D':
      updateDelay();
      break;
    case 'B':
      debug("Got loop begin command");
      started = true;
      break;
    case 'E':
      debug("Got loop end command");
      started = false;
      break;
    case 'T':
      debug("Got read date/time command");
      parseDateTime();
      break;
    case 'L':
      updateNumberOfLeds();
      break;
    default:
      Serial.print("# Unknown command ");
      Serial.print((char)command);
      Serial.println(" ignoring it");
      break;
  }
 readEndline();

}
