#ifndef LIGHT_SENSOR_CONSTANTS_H
#define LIGHT_SENSOR_CONSTANTS_H

#include <Arduino.h>

/*******************************************************
 * This file contains constants that may be changed if *
 * we change the underliying hardware, such as         *
 * voltages values, pin numbers etc                    *
 *******************************************************/


/*******************
 *  LED Management *
 *******************/


/*
 * Possible values
 * 0 - Led will be turned on only when needed
 * 1 - Led is set to be always on, even when not reading the values
 */
#define LED_ALWAYS_ON 1

/* Led for light-through measurement */
#define LED_PD1_PIN 6
/* Led for the reflection measurement */
#define LED_PD2_PIN 7
/* Pin to power the light-through measurement pd */
#define PD1_POWER_PIN 10
/* Pin to power the reflection measurement pd */
#define PD2_POWER_PIN 11
/* pd for light-through measurement */
#define PD1_PIN A0
/* pd for reflection measurement */
#define PD2_PIN A1
/* DHT Umidity sensor pin */
#define DHT_PIN 5
/* Temperature Measurement pin */
#define TEMPERATURE_PIN 4


/* 
 * Value is 8bit (1024 values), reference voltage is 1.1V
 * This means that the actual output in mV is
 * x * (1.100 / 1024) = x * 1.07422
 * 
 */
#define MV_PER_STEP 1.07422

/*******************************************************
 * Number of times that we read the light sensor value *
 * to compute the mean value                           *
 *******************************************************/
#define LIGHT_SENSOR_N_READS 1000



/************************
 * Loop management      *
 * The values are in ms *
 ************************/
#define NORMAL_LOOP_WAIT 600000
#define CALIBRATION_LOOP_WAIT 10000


/*********************************************************
 * Different values for the pins depending on the setup  *
 * do not modify them, you shouldn't need to             *
 *********************************************************/

static const int ledPins[2] =             { LED_PD1_PIN,      LED_PD2_PIN           };
static const int pdPins[2] =           { PD1_PIN,          PD2_PIN               };
static const int pdPowerPins[2] =      { PD1_POWER_PIN,    PD2_POWER_PIN         };
static const char* diodeDescription[2] =  { "PD1",            "PD2"                 };

/*
 * Possible values for the loop delay
 */
enum WAIT_CHOICE { NORMAL_LOOP=0, CALIBRATION_LOOP=1 };
static const long int waits[2] =          { NORMAL_LOOP_WAIT, CALIBRATION_LOOP_WAIT };



#endif

