/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package it.pastella.sensormonitor;

import com.google.inject.Guice;
import com.google.inject.Injector;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXMLLoader;
import javafx.fxml.JavaFXBuilderFactory;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;
import javafx.util.BuilderFactory;
import javafx.util.Callback;

/**
 *
 * @author mrighele
 */
public class MainApp extends Application {

    Injector injector = Guice.createInjector(new AppModule());

    private static void setupLogging() throws IOException {
        String logFile = System.getProperty("java.util.logging.config.file");
        if (logFile == null) {
            System.out.println("No log file, using standard properties");
            LogManager.getLogManager().readConfiguration(MainApp.class.getClassLoader().getResourceAsStream("logging.properties"));
        } else {
            LogManager.getLogManager().readConfiguration(new FileInputStream(logFile));
        }
    }

    @Override
    public void start(Stage stage) throws Exception {
        setupLogging();
        Logger.getGlobal().setLevel(Level.FINE);
        BuilderFactory builderFactory = new JavaFXBuilderFactory();
        Callback<Class<?>, Object> guiceControllerFactory = injector::getInstance;

        FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/MainView.fxml"), null, builderFactory, guiceControllerFactory);
        Parent root = loader.load();

        Scene scene = new Scene(root);
        scene.getStylesheets().add("/styles/Styles.css");

        stage.setTitle("Sensor Monitoring Tool");
        stage.setScene(scene);

        stage.setOnCloseRequest(new EventHandler<WindowEvent>() {
            @Override
            public void handle(WindowEvent t) {
                Platform.exit();
                System.exit(0);
            }
        });
        stage.show();
    }

    /**
     * The main() method is ignored in correctly deployed JavaFX application. main() serves only as fallback in case the
     * application can not be launched through deployment artifacts, e.g., in IDEs with limited FX support. NetBeans
     * ignores main().
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        launch(args);
    }

}
