/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package it.pastella.sensormonitor.controller;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import it.pastella.sensormonitor.service.DeviceReader;
import it.pastella.sensormonitor.service.DeviceReader.ColumnValue;
import it.pastella.sensormonitor.service.IDevice;
import it.pastella.sensormonitor.service.IDeviceFactory;
import it.pastella.sensormonitor.service.impl.CurrentTimeScheduler;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.ListProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.NumberAxis;
import javafx.scene.chart.XYChart;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.stage.FileChooser;
import javafx.stage.Window;
import javafx.util.Duration;
import javax.inject.Inject;

/**
 *
 * @author mrighele
 */
public class MainViewController implements Initializable {

    private final Logger log = Logger.getLogger(getClass().getCanonicalName());
    /*
    * FXML Elements
     */
    @FXML
    private Label label;

    @FXML
    private ChoiceBox<String> deviceChoiceBox;

    @FXML
    private Button connectButton;

    @FXML
    private Label messageLabel;

    @FXML
    private RadioButton led1Radio;

    @FXML
    private RadioButton led2Radio;

    @FXML
    private TableView dataTable;

    @FXML
    private LineChart<Date, Number> pd1ValueChart;
    @FXML
    private LineChart<Date, Number> pd1TemperatureChart;
    @FXML
    private LineChart<Date, Number> pd1HumidityChart;

    @FXML
    private LineChart<Date, Number> pd2ValueChart;
    @FXML
    private LineChart<Date, Number> pd2TemperatureChart;
    @FXML
    private LineChart<Date, Number> pd2HumidityChart;

    private IDevice.Command nLeds = IDevice.Command.LEDS1;
    private IDevice.Command delay = IDevice.Command.CALIBRATION_DELAY;

    @FXML
    private void handleConnectButtonAction(ActionEvent event) {
        log.log(Level.FINE, "Connect button clicked");
        switch (getStatus()) {
        case DISCONNECTED:
        case ERROR:
            connect();
            break;
        case CONNECTED:
            disconnect();
            break;
        default:
            throw new IllegalArgumentException("Invalid status " + getStatus());
        }
    }

    @FXML
    private void handleRefreshAction(ActionEvent event) {
        reloadDeviceList();
    }

    @FXML
    private void handleUpdateTimeAction(ActionEvent event) {
        LocalDateTime dt = LocalDateTime.now();
        device.updateTime(dt);
    }

    @FXML
    private void handleCalibrationDelaySelected(ActionEvent event) {
        log.fine("Calibration delay selected");
        delay = IDevice.Command.CALIBRATION_DELAY;
        device.sendCommand(IDevice.Command.CALIBRATION_DELAY);
    }

    @FXML
    private void handleNormalDelaySelected(ActionEvent event) {
        log.fine("Normal delay selected");
        delay = IDevice.Command.NORMAL_DELAY;
        device.sendCommand(IDevice.Command.NORMAL_DELAY);

    }

    private void connect() {
        log.log(Level.FINE, "Connecting");
        this.device = deviceFactory.getDevice(this.getDeviceName());
        try {
            this.deviceReader = new DeviceReader(device, logFile.get());
        } catch (Exception e) {
            log.severe(e.toString());
            setStatus(IDevice.ConnectionStatus.ERROR);
            return;
        }
        deviceReader.setOnRunning((event) -> {
            log.fine("Task running");
            this.setStatus(IDevice.ConnectionStatus.CONNECTED);
        });
        deviceReader.setOnFailed((e) -> {
            log.fine("Task failed: " + e.getSource().getException());
            this.setStatus(IDevice.ConnectionStatus.ERROR);
        });
        deviceReader.setOnCancelled((event) -> {
            log.fine("Task canceled");
            this.setStatus(IDevice.ConnectionStatus.DISCONNECTED);
        });
        this.setStatus(IDevice.ConnectionStatus.CONNECTING);
        this.lastDeviceOutputProperty().bind(deviceReader.valueProperty());
        this.lastDeviceOutputProperty().addListener((observable, oldValue, newValue) -> {
            List<ColumnValue> value = (List<ColumnValue>) newValue;
            updateDisplay(value);
            updateChart(value);
            saveData(value);
        });

        resetDataTable();
        deviceReader.start();
        try {
            // Send initial set of commands
            Thread.sleep(2000);
        } catch (InterruptedException ex) {
            Logger.getLogger(MainViewController.class.getName()).log(Level.SEVERE, null, ex);
        }
        device.sendCommand(delay);
        device.sendCommand(nLeds);
        device.sendCommand(IDevice.Command.BEGIN);
    }

    private void disconnect() {
        log.log(Level.FINE, "Disconnect");
        this.setStatus(IDevice.ConnectionStatus.DISCONNECTING);
        deviceReader.cancel();
    }

    @FXML
    private void handleChooseLogFileButtonAction(ActionEvent event) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Select Log File");
        Window window = deviceChoiceBox.getScene().getWindow();
        File file = fileChooser.showSaveDialog(window);
        setLogFile(file.getAbsolutePath());
    }

    @FXML
    private void handleDeviceChoiceBoxAction(ActionEvent event) {
        connectButton.setDisable(deviceChoiceBox.getValue() == null);

        setDeviceName(deviceChoiceBox.getValue());
    }

    @FXML
    private void handle1LedSelected(ActionEvent event) {
        nLeds = IDevice.Command.LEDS1;
        if (this.getStatus().equals(IDevice.ConnectionStatus.CONNECTED)) {
            device.sendCommand(IDevice.Command.LEDS1);
        }
    }

    @FXML
    private void handle2LedSelected(ActionEvent event) {
        nLeds = IDevice.Command.LEDS2;
        if (this.getStatus().equals(IDevice.ConnectionStatus.CONNECTED)) {
            device.sendCommand(IDevice.Command.LEDS2);
        }
    }

    @FXML
    private void handleTestModeSelected(ActionEvent event) {
        nLeds = IDevice.Command.LEDS_TEST;
        if (this.getStatus().equals(IDevice.ConnectionStatus.CONNECTED)) {
            device.sendCommand(IDevice.Command.LEDS_TEST);
        }
    }

    /**
     * Various services
     */
    private final IDeviceFactory deviceFactory;
    private IDevice device;
    private DeviceReader deviceReader;

    @Inject
    public MainViewController(IDeviceFactory deviceFactory) {
        this.deviceFactory = deviceFactory;
    }

    private void reloadDeviceList() {
        Collection<String> values = deviceFactory.getDeviceList();
        ObservableList<String> devices = FXCollections.observableArrayList(values);
        deviceChoiceBox.setItems(devices);
        if (values.size() == 1) {
            deviceChoiceBox.setValue(values.iterator().next());
        } else {
            deviceChoiceBox.setValue(null);
        }
    }

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        initializeCharts();

        reloadDeviceList();
        status.addListener((observable) -> {
            log.fine("Status changed to: " + getStatus());
            switch (getStatus()) {
            case CONNECTED:
                connected.set(true);
                connectButton.setText("Disconnect");
                connectButton.setDisable(false);
                led2Radio.setDisable(true);
                led1Radio.setDisable(true);
                break;
            case DISCONNECTED:
            case ERROR:
                connected.set(false);
                connectButton.setText("Connect");
                connectButton.setDisable(false);
                led2Radio.setDisable(false);
                led1Radio.setDisable(false);
                break;
            case CONNECTING:
                connected.set(false);
                connectButton.setText("Disconnect");
                connectButton.setDisable(true);
                led2Radio.setDisable(true);
                led1Radio.setDisable(true);
                break;
            case DISCONNECTING:
                connectButton.setDisable(true);
                led2Radio.setDisable(true);
                led1Radio.setDisable(true);
                break;
            default:
                throw new IllegalArgumentException("Unknown status " + getStatus());
            }
        });

        dataTable.setItems(this.getItems());
        lastDeviceOutputProperty().addListener((observable, oldValue, newValue) -> {
            log.fine("New output from device: " + newValue);
        });
        CurrentTimeScheduler scheduler = new CurrentTimeScheduler();
        scheduler.setPeriod(Duration.seconds(1));
        int x = 2;
        scheduler.setOnSucceeded((event) -> {
            LocalDateTime currentDate = (LocalDateTime) event.getSource().getValue();
            this.setHostDateTime(currentDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        });
        scheduler.start();
    }

    /*
    * FXML Properties
     */
    private final StringProperty logFile = new SimpleStringProperty();

    public String getLogFile() {
        return logFile.get();
    }

    public void setLogFile(String value) {
        log.fine("Setting log file to " + value);
        logFile.set(value);
    }

    public StringProperty logFileProperty() {
        return logFile;
    }
    private final StringProperty deviceName = new SimpleStringProperty();

    public String getDeviceName() {
        return deviceName.get();
    }

    public void setDeviceName(String value) {
        log.fine("Setting device name to " + value);
        deviceName.set(value);
    }

    public StringProperty deviceNameProperty() {
        return deviceName;
    }
    private final ObjectProperty<IDevice.ConnectionStatus> status = new SimpleObjectProperty<>(IDevice.ConnectionStatus.DISCONNECTED);

    public IDevice.ConnectionStatus getStatus() {
        return status.get();
    }

    public void setStatus(IDevice.ConnectionStatus value) {
        log.fine("Setting status to " + value);
        status.set(value);
    }

    public ObjectProperty statusProperty() {
        return status;
    }
    private final ListProperty<DeviceReader.ColumnValue> lastDeviceOutput = new SimpleListProperty<>();

    public ObservableList getLastDeviceOutput() {
        return lastDeviceOutput.get();
    }

    public void setLastDeviceOutput(ObservableList value) {
        lastDeviceOutput.set(value);
    }

    public ListProperty lastDeviceOutputProperty() {
        return lastDeviceOutput;
    }

    /**
     * Return the background color (as a CSS string) for the given column
     *
     * @param columnName
     * @return
     */
    private String getColumnColor(String columnName) {
        ColumnInformation ci = ColumnInformation.getByName(columnName);
        if (ci != null) {
            return ci.getBackgroundColor();
        } else {
            return null;
        }
    }

    private void updateDisplay(List<ColumnValue> newValue) {
        for (ColumnValue cv : newValue) {
            // FIXME: This assumes that the indexes are ordered
            int index = cv.getIndex();
            if (index >= this.dataTable.getColumns().size()) {
                String label = cv.getKey();
                if (cv.getUm() != null && !cv.getUm().equals("")) {
                    label = label + " [" + cv.getUm() + "]";
                }
                TableColumn<List<ColumnValue>, String> column = new TableColumn<>(label);
                String color = getColumnColor(cv.getKey());
                log.info("Index: " + index + " for " + cv.getKey());
                column.setCellValueFactory((param) -> new SimpleStringProperty(param.getValue().get(index).getValue()));
                column.getStyleClass().add(cv.getKey());
                this.dataTable.getColumns().add(column);
            }

        }

        this.getItems().add(newValue);
    }

    private void saveData(List<ColumnValue> value) {
        if (this.getLogFile() != null) {
            // FIXME: Not good to do this on the UI thread, but should be so quick that the user shouldn't realize it
            File f = new File(getLogFile());
            try (PrintWriter pw = new PrintWriter(new FileOutputStream(f, true))) {
                boolean first = true;
                for (ColumnValue cv : value) {
                    if (!first) {
                        pw.append(", ");
                    } else {
                        first = false;
                    }
                    pw.write(cv.getKey());
                    pw.write(", ");
                    pw.write(cv.getValue());
                    pw.write(", ");
                    pw.write(cv.getUm());
                }
                pw.write("\n");
            } catch (FileNotFoundException ex) {
                Logger.getLogger(MainViewController.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private final BooleanProperty connected = new SimpleBooleanProperty();

    public boolean isConnected() {
        return connected.get();
    }

    public void setConnected(boolean value) {
        connected.set(value);
    }

    public BooleanProperty connectedProperty() {
        return connected;
    }
    private final ListProperty<List<ColumnValue>> items = new SimpleListProperty<>(FXCollections.observableArrayList());

    public ObservableList getItems() {
        return items.get();
    }

    public void setItems(ObservableList value) {
        items.set(value);
    }

    public ListProperty< List<ColumnValue>> itemsProperty() {
        return items;
    }

    private void resetDataTable() {
        items.clear();
        this.dataTable.getColumns().clear();
    }
    private final StringProperty hostDateTime = new SimpleStringProperty();

    public String getHostDateTime() {
        return hostDateTime.get();
    }

    public void setHostDateTime(String value) {
        hostDateTime.set(value);
    }

    public StringProperty hostDateTimeProperty() {
        return hostDateTime;
    }

    private Multimap<String, XYChart.Series<Date, Number>> seriesMap;

    private XYChart.Series<Date, Number> pd1ValueSeries;
    private XYChart.Series<Date, Number> pd1HumiditySeries;
    private XYChart.Series<Date, Number> pd1TemperatureSeries;
    private XYChart.Series<Date, Number> pd2ValueSeries;
    private XYChart.Series<Date, Number> pd2HumiditySeries;
    private XYChart.Series<Date, Number> pd2TemperatureSeries;

    private void initializeCharts() {
        seriesMap = ArrayListMultimap.create();

        pd1ValueSeries = new XYChart.Series<>();
        pd1ValueSeries.setName("PD1");
        pd1ValueChart.getData().add(pd1ValueSeries);
        pd1HumiditySeries = new XYChart.Series<>();
        pd1HumiditySeries.setName("Humidity");
        pd1HumidityChart.getData().add(pd1HumiditySeries);

        pd1TemperatureSeries = new XYChart.Series<>();
        pd1TemperatureSeries.setName("Temperature");
        pd1TemperatureChart.getData().add(pd1TemperatureSeries);

        pd2ValueSeries = new XYChart.Series<>();
        pd2ValueSeries.setName("PD2");
        pd2ValueChart.getData().add(pd2ValueSeries);

        pd2HumiditySeries = new XYChart.Series<>();
        pd2HumiditySeries.setName("Humidity");
        pd2HumidityChart.getData().add(pd2HumiditySeries);

        pd2TemperatureSeries = new XYChart.Series<>();
        pd2TemperatureSeries.setName("Temperature");
        pd2TemperatureChart.getData().add(pd2TemperatureSeries);

        seriesMap.put("PD1", pd1ValueSeries);
        seriesMap.put("PD2", pd2ValueSeries);

        // Case 1 led
        seriesMap.put("Hum", pd1HumiditySeries);
        seriesMap.put("Hum", pd2HumiditySeries);
        seriesMap.put("Temp", pd1TemperatureSeries);
        seriesMap.put("Temp", pd2TemperatureSeries);

    }

    private void updateChart(List<ColumnValue> values) {
        Date date = new Date();
        for (ColumnValue cv : values) {
            if (cv.getKey().equals("Date")) {
                String dateValue = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                cv.setValue(dateValue);
            }
        }
//        Date date = null;
//        for (ColumnValue cv : values) {
//            if (cv.getKey().equals("Date")) {
//                date = Date.from(LocalDateTime.parse(cv.getValue().trim(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).atZone(ZoneId.systemDefault()).toInstant());
//            }
//        }
//        if (date == null) {
//            return;
//        }
        for (ColumnValue cv : values) {
            for (XYChart.Series<Date, Number> series : seriesMap.get(cv.getKey())) {
                Double value;
                try {
                    value = Double.valueOf(cv.getValue());
                } catch (NumberFormatException e) {
                    value = null;
                }
                if (value != null) {
                    series.getData().add(new XYChart.Data<>(date, value));
                    NumberAxis axis = (NumberAxis) series.getChart().getYAxis();
                    if (axis.isAutoRanging()) {
                        axis.setUpperBound(value + 1);
                        axis.setLowerBound(value - 1);
                        axis.setAutoRanging(false);
                    } else {
                        double newMin = Math.floor(Math.min(value, axis.getLowerBound()));
                        double newMax = Math.ceil(Math.max(value, axis.getUpperBound()));

                        axis.setUpperBound(newMax);
                        axis.setLowerBound(newMin);
                        axis.setTickUnit((newMax - newMin) / 10);
                    }
                }
            }
        }
    }

}
