/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package it.pastella.sensormonitor.service;

import java.io.IOException;
import java.util.List;
import java.util.logging.Logger;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Service;
import javafx.concurrent.Task;

/**
 *
 * @author mrighele
 */
public class DeviceReader extends Service<List<DeviceReader.ColumnValue>> {

	public static class ColumnValue {

		public ColumnValue(int index, String key, String value, String um) {
			this.index = index;
			this.key = key;
			this.value = value;
			this.um = um;
		}

		private int index;
		private String key;
		private String value;
		private String um;

		/**
		 * Get the position of the column in the grid
		 */
		public int getIndex() {
			return index;
		}

		public void setIndex(int index) {
			this.index = index;
		}

		/**
		 * Get the name of the column
		 *
		 * @return
		 */
		public String getKey() {
			return key;
		}

		public void setKey(String key) {
			this.key = key;
		}

		/**
		 * Get the value of the specific row
		 *
		 * @return
		 */
		public String getValue() {
			return value;
		}

		public void setValue(String value) {
			this.value = value;
		}

		/**
		 * Get the unit of measurement to be used with the value
		 *
		 * @return
		 */
		public String getUm() {
			return um;
		}

		public void setUm(String um) {
			this.um = um;
		}

	}

	private IDevice device;
	private final Logger log = Logger.getLogger(getClass().getCanonicalName());

	public DeviceReader(IDevice device, String output) throws IOException {
		log.fine("Creating device reader");
		this.device = device;
		if (output != null) {
			log.fine("Using " + output + " as a log file");
		} else {
			log.fine("No log file specified!");

		}
	}

	@Override
	protected Task<List<ColumnValue>> createTask() {
		return new Task< List<ColumnValue>>() {
			@Override
			protected List<ColumnValue> call() throws Exception {
				while (!Thread.interrupted()) {
					ObservableList<ColumnValue> result = FXCollections.observableArrayList();
					log.finest("Getting next line from serial port");
					String line = null;
					while (line == null || line.startsWith("#")) {
						try {
							line = device.readLine();
							if (line != null) {
								log.finest(">>> " + line);
							}
						} catch (IOException e) {
							log.warning("SerialPort handling sucks...");
							Thread.sleep(100);
						}
					}
					if (Thread.interrupted()) {
						break;
					}
					log.fine("Got line " + line);
					if (line.startsWith("N:") == false) {
						log.finest("Got a partial line, skip it");
						continue;
					}
					String[] tokens = line.split(",");
					for (int i = 0; i + 2 < tokens.length; i += 3) {
						int index = i / 3;
						String key = tokens[i];
						String value = tokens[i + 1];
						String um = tokens[i + 2];
						result.add(new ColumnValue(index, key, value, um));
					}
					updateValue(result);
				}
				log.info("DeviceReader thread interrupted, closing device");
				device.close();
				return null;
			}

		};
	}
}
