/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package it.pastella.sensormonitor.service.impl;

import gnu.io.NRSerialPort;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import it.pastella.sensormonitor.service.IDevice;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author mrighele
 */
public class Device implements IDevice {

	private final Logger log = Logger.getLogger(getClass().getSimpleName());
	private final NRSerialPort serialPort;
	private final BufferedReader input;
	private final DataOutputStream output;
	private StringBuffer buffer = new StringBuffer();

	Device(NRSerialPort serialPort) {
		this.serialPort = serialPort;
		try {
			serialPort.getSerialPortInstance().setSerialPortParams(9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			serialPort.getSerialPortInstance().disableReceiveTimeout();
			serialPort.getSerialPortInstance().enableReceiveThreshold(1);

		} catch (UnsupportedCommOperationException e) {
			log.log(Level.SEVERE, "Cannot configure serial port: " + e);
		}

		input = new BufferedReader(new InputStreamReader(serialPort.getInputStream()));
		output = new DataOutputStream(serialPort.getOutputStream());
	}

	@Override
	public String readLine() throws IOException {
		return input.readLine();
	}

	private void sendString(String s) throws IOException {
		log.severe("Sending command " + s);
		ByteBuffer byteBuffer = Charset.forName("UTF-8").encode(s);
		output.write(byteBuffer.array());
	}

	@Override
	public void updateTime(LocalDateTime dateTime) {
		String date = dateTime.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);
		try {
			sendString("T" + date + "\n");
		} catch (IOException ex) {
			Logger.getLogger(Device.class.getName()).log(Level.SEVERE, null, ex);
		}

	}

	@Override
	public void sendCommand(Command command) {
		try {
			switch (command) {
			case CALIBRATION_DELAY:
				sendString("DC\n");
				break;
			case NORMAL_DELAY:
				sendString("DN\n");
				break;
			case BEGIN:
				sendString("B\n");
				break;
			case END:
				sendString("E\n");
				break;
			case LEDS1:
				sendString("L1\n");
				break;
			case LEDS2:
				sendString("L2\n");
				break;
			case LEDS_TEST:
				sendString("LT\n");
				break;
			default:
				log.warning("Unknown command " + command);
			}
		} catch (IOException ex) {
			log.log(Level.SEVERE, null, ex);
		}
	}

	@Override
	public void close() {
		serialPort.disconnect();
	}
}
